package com.tzld.piaoquan.tencentad.service.impl;

import com.tzld.piaoquan.tencentad.common.base.CommonResponse;
import com.tzld.piaoquan.tencentad.common.base.ExceptionCodeEnum;
import com.tzld.piaoquan.tencentad.common.enums.AdVideoImageStatusEnum;
import com.tzld.piaoquan.tencentad.common.enums.DimensionSizeEnum;
import com.tzld.piaoquan.tencentad.common.enums.SimilarityCodeEnum;
import com.tzld.piaoquan.tencentad.dao.mapper.AdVideoImageMapper;
import com.tzld.piaoquan.tencentad.model.po.AdMuseTask;
import com.tzld.piaoquan.tencentad.model.po.AdVideoImage;
import com.tzld.piaoquan.tencentad.model.po.AdVideoImageExample;
import com.tzld.piaoquan.tencentad.model.vo.AdVideoImageParam;
import com.tzld.piaoquan.tencentad.model.vo.AdVideoImageVo;
import com.tzld.piaoquan.tencentad.service.AdAccountService;
import com.tzld.piaoquan.tencentad.service.AdMuseTaskService;
import com.tzld.piaoquan.tencentad.service.AdVideoImageService;
import com.tzld.piaoquan.tencentad.service.TencentService;
import com.tzld.piaoquan.tencentad.utils.DateUtils;
import com.tzld.piaoquan.tencentad.utils.OSSUploader;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static com.tzld.piaoquan.tencentad.common.base.ExceptionCodeEnum.PARAM_ERROR;

@Slf4j
@Service
public class AdVideoImageServiceImpl implements AdVideoImageService {

    @Value("${image_path}")
    private String imagePath;

    @Autowired
    private AdVideoImageMapper adVideoImageMapper;

    @Autowired
    private TencentService tencentService;

    @Autowired
    private AdMuseTaskService adMuseTaskService;

    @Autowired
    private AdAccountService accountService;

    @Override
    public boolean addAdVideoImage(Long adVideoId, String url) {
        try {
            AdVideoImage adVideoImage = new AdVideoImage();
            adVideoImage.setAdVideoId(adVideoId);
            adVideoImage.setImageUrl(url);
            adVideoImageMapper.insertSelective(adVideoImage);
            return true;
        } catch (Exception e) {
            log.error("addAdVideoImage error {}", e.getMessage());
        }
        return false;
    }

    @Override
    public CommonResponse<List<AdVideoImageVo>> getAdVideoImageVoList(Long adVideoId) {
        AdVideoImageExample example = new AdVideoImageExample();
        example.createCriteria().andAdVideoIdEqualTo(adVideoId).andIsDeleteEqualTo(0);
        List<AdVideoImage> adVideoImages = adVideoImageMapper.selectByExample(example);
        List<AdVideoImageVo> adVideoImageVos = new ArrayList<>();
        for (AdVideoImage adVideoImage : adVideoImages) {
            AdVideoImageVo adVideoImageVo = new AdVideoImageVo();
            BeanUtils.copyProperties(adVideoImage, adVideoImageVo);
            adVideoImageVo.setCreateTime(DateUtils.getDateString(adVideoImage.getCreateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adVideoImageVo.setUpdateTime(DateUtils.getDateString(adVideoImage.getUpdateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adVideoImageVos.add(adVideoImageVo);
        }
        return CommonResponse.success(adVideoImageVos);
    }

    public CommonResponse<Void> updateAdVideoImage(AdVideoImageParam adVideoImageParam) {
        if (adVideoImageParam == null || CollectionUtils.isEmpty(adVideoImageParam.getIds())) {
            return CommonResponse.create(PARAM_ERROR.getCode(), PARAM_ERROR.getMsg());
        }
        String accessToken = accountService.getAccessToken(adVideoImageParam.getAccountId());
        if (StringUtils.isEmpty(accessToken)) {
            return CommonResponse.create(500, "accessToken获取失败，确认账号无误后重试");
        }
        for (Long id : adVideoImageParam.getIds()) {
            AdVideoImage adVideoImage = adVideoImageMapper.selectByPrimaryKey(id);
            if (adVideoImage == null) {
                continue;
            }
            String imageId = uploadImageToTencent(adVideoImageParam.getAccountId(), accessToken, adVideoImage);
            if (StringUtils.isNotEmpty(imageId)) {
                adMuseTaskService.addMuseTask(adVideoImageParam.getAccountId(), accessToken, adVideoImageParam, adVideoImage);
            }
        }

        return CommonResponse.success();
    }

    //上传图片到腾讯
    @Override
    public String uploadImageToTencent(String account, String accessToken, AdVideoImage adVideoImage) {
        if (StringUtils.isNotEmpty(adVideoImage.getTencentImageId())) {
            return adVideoImage.getTencentImageId();
        }
        String imageId = tencentService.uploadImage(account, accessToken, adVideoImage.getImageUrl());
        adVideoImage.setTencentImageId(imageId);
        adVideoImageMapper.updateByPrimaryKeySelective(adVideoImage);
        return imageId;

    }

    @Override
    public CommonResponse<Void> uploadMuseImage(MultipartFile file, Long adVideoId) {
        if (adVideoId == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，adVideoId不能为空");
        }
        // 创建上传目录
        String fileName = file.getOriginalFilename();
        if (fileName == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，文件名无效");
        }
        File uploadDir = new File(imagePath);
        if (!uploadDir.exists()) {
            uploadDir.mkdirs(); // 创建目录
        }
        String path = imagePath + "/" + fileName;
        // 保存文件
        File destinationFile = new File(uploadDir, fileName);
        try {
            file.transferTo(destinationFile);
        } catch (IOException e) {
            return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "系统异常，上传失败");
        }
        String url = OSSUploader.uploadToOSS(path);
        if (StringUtils.isNotEmpty(url)) {
            if (addAdVideoImage(adVideoId, url)) {
                return CommonResponse.success();
            }
        }
        return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "系统异常，上传失败");
    }

    @Override
    public AdVideoImage getAdVideoImageById(Long adVideoImageId) {
        return adVideoImageMapper.selectByPrimaryKey(adVideoImageId);
    }


}
