package com.tzld.piaoquan.tencentad.utils;


import com.tzld.piaoquan.tencentad.model.bo.ImageParam;
import lombok.extern.slf4j.Slf4j;
import sun.misc.BASE64Encoder;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class ImageUtils {

    public static void main(String[] args) {
        ImageParam imageParam = new ImageParam();
        imageParam.setFont(new Font("微软雅黑", Font.PLAIN, 48));
        imageParam.setColor(new Color(102, 102, 102));
        imageParam.setText("这是一个非常长的字幕");
        System.out.println(writeImageFromUrl("http://videomaker-30171.sz.gfp.tencent-cloud.com/381af314920e2d856f283ccf237eaec05917018d", "/Users/shimeng/Desktop/image_path/sss.jpg", imageParam));
    }

    public static String writeImageFromUrl(String srcImgUrl, String targetImgPath, ImageParam imageParam) {
        try {
            // 读取原图片信息
            URL url = new URL(srcImgUrl); // 通过 URL 创建
            Image srcImg = ImageIO.read(url); // 从 URL 读取图片
            int imageWidth = srcImg.getWidth(null); // 获取图片的宽
            int imageHeight = srcImg.getHeight(null); // 获取图片的高

            // 创建一个新的图片，宽度和高度加上文字的高度
            BufferedImage newImage = new BufferedImage(imageWidth, imageHeight, BufferedImage.TYPE_INT_RGB);
            Graphics2D g2d = newImage.createGraphics();

            // 绘制原始图片
            g2d.drawImage(srcImg, 0, 0, null);

            // 设置字体和颜色
            g2d.setFont(new Font("Arial", Font.BOLD, 48)); // 字体和大小
            g2d.setColor(Color.BLACK); // 字体颜色

            // 计算文字的宽度和位置
            FontMetrics fontMetrics = g2d.getFontMetrics();
            int textWidth = fontMetrics.stringWidth(imageParam.getText());
            int x = (imageWidth - textWidth) / 2; // 中间对齐
            int y = imageHeight - 50; // 在图片下方添加文字

            // 添加字幕
            g2d.drawString(imageParam.getText(), x, y);
            g2d.dispose();
            ImageIO.write(newImage, "jpg", new File(targetImgPath));
            String ossPath = OSSUploader.uploadToOSS(targetImgPath);
            Files.delete(Paths.get(targetImgPath));
            return ossPath;
        } catch (Exception e) {
            log.error("writeImageFromUrl error", e);
        }
        return null;
    }


    /**
     * 图片URL转Base64编码
     *
     * @param imgUrl 图片URL
     * @return Base64编码
     */
    public static String imageUrlToBase64(String imgUrl) {
        URL url = null;
        InputStream is = null;
        ByteArrayOutputStream outStream = null;
        HttpURLConnection httpUrl = null;

        try {
            url = new URL(imgUrl);
            httpUrl = (HttpURLConnection) url.openConnection();
            httpUrl.connect();
            httpUrl.getInputStream();

            is = httpUrl.getInputStream();
            outStream = new ByteArrayOutputStream();

            //创建一个Buffer字符串
            byte[] buffer = new byte[1024];
            //每次读取的字符串长度，如果为-1，代表全部读取完毕
            int len = 0;
            //使用输入流从buffer里把数据读取出来
            while ((len = is.read(buffer)) != -1) {
                //用输出流往buffer里写入数据，中间参数代表从哪个位置开始读，len代表读取的长度
                outStream.write(buffer, 0, len);
            }

            // 对字节数组Base64编码
            return encode(outStream.toByteArray());
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (is != null) {
                    is.close();
                }
                if (outStream != null) {
                    outStream.close();
                }
                if (httpUrl != null) {
                    httpUrl.disconnect();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return null;
    }

    /**
     * 图片转字符串
     *
     * @param image 图片Buffer
     * @return Base64编码
     */
    public static String encode(byte[] image) {
        BASE64Encoder decoder = new BASE64Encoder();
        return replaceEnter(decoder.encode(image));
    }

    /**
     * 字符替换
     *
     * @param str 字符串
     * @return 替换后的字符串
     */
    public static String replaceEnter(String str) {
        String reg = "[\n-\r]";
        Pattern p = Pattern.compile(reg);
        Matcher m = p.matcher(str);
        return m.replaceAll("");
    }

}


